'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});

var _slicedToArray = (function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i['return']) _i['return'](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError('Invalid attempt to destructure non-iterable instance'); } }; })();

exports['default'] = render;

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

function escapeMarkdown(input) {
	return [

	// escaping symbols: # * ( ) [ ] _
	[/([#\*\(\)\[\]\_\/`])/g, '\\$1'],

	// escaping less and more signs
	[/\</g, '&lt;'], [/\>/g, '&gt;']].reduce(function (input, _ref) {
		var _ref2 = _slicedToArray(_ref, 2);

		var replaceFrom = _ref2[0];
		var replaceTo = _ref2[1];
		return input.replace(replaceFrom, replaceTo);
	}, input);
}

function renderEntityMention(data) {
	return '[@' + escapeMarkdown(data.screen_name) + '](https://twitter.com/' + data.screen_name + ' "' + data.name + '")';
}

function renderEntityMedia(data) {
	return '[' + escapeMarkdown(data.display_url) + '](' + data.url + ')';
}

function renderEntityHashtag(data) {
	return '[#' + escapeMarkdown(data.text) + '](https://twitter.com/search?q=%23' + data.text + ')';
}

function renderEntityUrl(data) {
	return '[' + escapeMarkdown(data.display_url) + '](' + data.url + ' "' + data.expanded_url + '")';
}

function renderEntity(type, data) {
	switch (type) {
		case 'user_mentions':
			return renderEntityMention(data);
		case 'media':
			return renderEntityMedia(data);
		case 'hashtags':
			return renderEntityHashtag(data);
		case 'urls':
			return renderEntityUrl(data);
		default:
			return null;
	}
}

function render(tweet) {
	var text = tweet.text;
	var _tweet$entities = tweet.entities;
	var entities = _tweet$entities === undefined ? {} : _tweet$entities;

	var replacements = [];

	Object.keys(entities).forEach(function (entityKey) {
		replacements.push.apply(replacements, _toConsumableArray(entities[entityKey].map(function (entity) {
			return [renderEntity(entityKey, entity), entity.indices[0], entity.indices[1]];
		})));
	});

	if (0 === replacements.length) {
		return escapeMarkdown(tweet.text);
	}

	var lastPos = text.length;
	var parts = replacements.sort(function (a, b) {
		return b[1] - a[1];
	}).map(function (replacement) {
		var output = [replacement[0]];

		if (replacement[2] < lastPos) {
			output.push(escapeMarkdown(text.substr(replacement[2], lastPos - replacement[2])));
		}

		lastPos = replacement[1];

		return output.join('');
	});

	if (lastPos > 0) {
		parts.push(escapeMarkdown(text.substr(0, lastPos)));
	}

	return parts.reverse().join('');
}

module.exports = exports['default'];
